searchNodes=[{"ref":"StateMachine.html","title":"StateMachine","type":"module","doc":"StateMachine package implements state machine abstraction. It supports Ecto out of the box and can work as both data structure and a process powered by gen_statem. Check out the article for motivation. Here&#39;s an example of a simple state machine created with this package: defmodule Cat do use StateMachine defstruct [:name, :state, hungry: true] defmachine field: :state do state :asleep state :awake state :playing state :eating, after_enter: &amp;Cat.feed_up/1 event :wake do transition from: :asleep, to: :awake end event :give_a_mouse do transition from: :awake, to: :playing, unless: &amp;Cat.hungry/1 transition from: :awake, to: :eating, if: &amp;Cat.hungry/1 transition from: :playing, to: :eating end event :pet do transition from: [:eating, :awake], to: :playing end event :sing_a_lullaby do transition from: :awake, to: :asleep transition from: :playing, to: :asleep end end def hungry(cat) do cat.hungry end def feed_up(cat) do {:ok, %{cat | hungry: false}} end end And later use it like this: cat = %Cat{name: &quot;Thomas&quot;, state: :asleep} {:ok, %Cat{state: :awake}} = Cat.trigger(cat, :wake) Features Validation of state machine definition at compile time Full support for callbacks (on states, events and transitions) and guards (on events and transitions) Optional payload can be supplied with the event One-line conversion to a state machine as a process (powered by gen_statem) With Ecto support activated every transition is wrapped in transaction With Ecto support activated the Ecto.Type implementation is generated automatically"},{"ref":"StateMachine.html#t:t/1","title":"StateMachine.t/1","type":"type","doc":""},{"ref":"StateMachine.Callback.html","title":"StateMachine.Callback","type":"module","doc":"Callback wraps a captured function, that can be called in a various points of a State Machine&#39;s lifecycle. Depending on return type (shape) of the callback, it can update the context or the model, or be ignored."},{"ref":"StateMachine.Callback.html#apply_callback/3","title":"StateMachine.Callback.apply_callback/3","type":"function","doc":"Applying a single callback. There are three types of callbacks supported: Unary A unary callback, receiving a model struct and that can be updated in current conetxt based on shape of the return: {:ok, model} — replaces model in the context {:error, e} — stops the transition with a given error any — doesn&#39;t have any effect on the context Binary A binary callback, receiving a model struct and a context. Either can be updated depending on the shape of the return: {:ok, context} — replaces context completely {:ok, model} — replaces model in the context {:error, e} — stops the transition with a given error any — doesn&#39;t have any effect on the context Side effects A type of callback that does not expect any input and potentially produces a side effect. Any return value is ignored, except for {:error, e} that stops the transition with a given error."},{"ref":"StateMachine.Callback.html#apply_chain/3","title":"StateMachine.Callback.apply_chain/3","type":"function","doc":"Applying a chain of callbacks. Application only happens if status hasn&#39;t been changed."},{"ref":"StateMachine.Callback.html#t:binary_t/1","title":"StateMachine.Callback.binary_t/1","type":"type","doc":""},{"ref":"StateMachine.Callback.html#t:side_effect_t/0","title":"StateMachine.Callback.side_effect_t/0","type":"type","doc":""},{"ref":"StateMachine.Callback.html#t:t/1","title":"StateMachine.Callback.t/1","type":"type","doc":""},{"ref":"StateMachine.Callback.html#t:unary_t/1","title":"StateMachine.Callback.unary_t/1","type":"type","doc":""},{"ref":"StateMachine.Context.html","title":"StateMachine.Context","type":"module","doc":"Context is a container for all metadata supporting the transition. Normally users should not have anything to do with Contexts, however some public functions might expose it. For example, user can (but discuraged) manipulate the Context in callbacks."},{"ref":"StateMachine.Context.html#build/2","title":"StateMachine.Context.build/2","type":"function","doc":"Builds a fresh context based on State Machine definition and a model struct."},{"ref":"StateMachine.Context.html#t:t/1","title":"StateMachine.Context.t/1","type":"type","doc":""},{"ref":"StateMachine.DSL.html","title":"StateMachine.DSL","type":"module","doc":"These macros help generating State Machine definition in a given Module."},{"ref":"StateMachine.DSL.html#define_gen_statem/0","title":"StateMachine.DSL.define_gen_statem/0","type":"macro","doc":"Experimental macro to generate GenStatem definition. See source..."},{"ref":"StateMachine.DSL.html#defmachine/2","title":"StateMachine.DSL.defmachine/2","type":"macro","doc":"Creates a main block for a State Machine definition. Compile time checks and validators help ensuring correct usage of this and other macros. use StateMachine defmachine field: :custom do # ... states, events, transitions end Options :field - what field in the model structure holds the state value. Default: :state. :repo - the Ecto Repo module. If provider, the support for Ecto is activated, including state getter/setters, Ecto.Type generation and custom trigger function. :ecto_type - name for Ecto.Type module. Generated inside of the module namespace. If you state maching is App.StateMachine, then Ecto.Type implementation will be accessible on App.StateMachine.(ecto_type). Default: StateType."},{"ref":"StateMachine.DSL.html#event/3","title":"StateMachine.DSL.event/3","type":"macro","doc":"Creates an Event record which encapsulates one or more Transitions. Conceptually an Event is an external signal to change the state. Events can be accompanied by Guards — additional conditions, allowing for implementing more complex logic than supported by theoretical state machines. Using guards allow for transitions from one to the multiple states, based on condition. defmachine do # for state defs, see `state` event :wake_up, if: &amp;Bedroom.slept_eight_hours?/1 do # for transition defs, see `transition` end end Options :before - run the callback before the event. :after - run the callback after the event. :if - positive guard, must return true to proceed. :unless - negative guard, must return false to proceed."},{"ref":"StateMachine.DSL.html#state/2","title":"StateMachine.DSL.state/2","type":"macro","doc":"Creates a State record with any atom as a name. Supports defining callbacks for before/after leaving/entering given state. For detailed description of Callbacks, see the module documentation, but currently only fully qualified function capture is supported: &amp;Module.fun/arity defmachine do state :sleeping, after_leave: &amp;Kitchen.brew_coffee/1 state :working, before_enter: [&amp;Commute.drive/1, &amp;Commute.grab_a_newspaper/1] end Options :before_leave - run the callback before leaving this state. :after_leave - run the callback after leaving this state. :before_enter - run the callback before entering this state. :after_enter - run the callback after entering this state."},{"ref":"StateMachine.DSL.html#transition/1","title":"StateMachine.DSL.transition/1","type":"macro","doc":"Defines a Transition, a path from one or more states to a single state. The same Event can contain all crazy combinations of transitions, but the resolution happens from top to bottom, from left to right. First matching transition will run. defmachine do # for state defs, see `state` event :wake_up do transition from: :sleeping, to: :sleeping, if: &amp;Calendar.weekend?/0, unless: &amp;Bedroom.slept_ten_hours?/1 transition from: :sleeping, to: :awake end end Options :before - run the callback before the transition. :after - run the callback after the transition. :if - positive guard, must return true to proceed. :unless - negative guard, must return false to proceed."},{"ref":"StateMachine.Ecto.html","title":"StateMachine.Ecto","type":"module","doc":"This addition makes StateMachine fully compatible with Ecto. State setter and getter are abstracted in order to provide a way to update a state in the middle of transition for a various types of models. With Ecto, we call change() |&gt; Repo.update. We also wrap every event in transaction, which is rolled back if transition fails to finish. This unlocks a lot of beautiful effects. For example, you can enqueue some tasks into db-powered queue in callbacks, and if transition failes, those tasks will naturally disappear. Usage To use Ecto, simply pass repo param to defmachine, you can optionally pass a name of the Ecto.Type implementation, that will be generated automatically under state machine namespace: defmodule EctoMachine do use StateMachine defmachine field: :state, repo: TestApp.Repo, ecto_type: CustomMod do state :resting state :working # ... end end In your schema you can refer to state type as EctoMachine.CustomMod, with ecto_type omitted it would generate EctoMachine.StateType. This custom type is needed to transparently use atoms as states."},{"ref":"StateMachine.Ecto.html#define_ecto_type/0","title":"StateMachine.Ecto.define_ecto_type/0","type":"macro","doc":"This macro defines an Ecto.Type implementation inside of StateMachine namespace. The default name is StateType, but you can supply any module name. The purpose of this is to be able to cast string into atom and back safely, validating it against StateMachine defition."},{"ref":"StateMachine.Event.html","title":"StateMachine.Event","type":"module","doc":"Event is a container for transitions. It is identified by name (atom) and can contain arbitrary number of transtitions. One important thing is that it&#39;s disallowed to have more than one unguarded transition from the state, since this would introduce a &quot;non-determinism&quot; (or rather just discard the latter transition). We loosely allow guarded transitions from the same state, but it doesn&#39;t guarantee anything: if guards always return true, we&#39;re back to where we were before."},{"ref":"StateMachine.Event.html#after_/1","title":"StateMachine.Event.after_/1","type":"function","doc":"Private function for running after_event callbacks."},{"ref":"StateMachine.Event.html#before/1","title":"StateMachine.Event.before/1","type":"function","doc":"Private function for running before_event callbacks."},{"ref":"StateMachine.Event.html#is_allowed?/2","title":"StateMachine.Event.is_allowed?/2","type":"function","doc":"Checks if the event is allowed in the current context. First it makes sure that all guards of the event return true, then it scans transitions for the matching one. Match is determined by the source state and passing of all guards as well."},{"ref":"StateMachine.Event.html#trigger/3","title":"StateMachine.Event.trigger/3","type":"function","doc":"This is an entry point for transition. By running this function with populated context, event and optional payload, you tell state machine to try to move to the next state. It returns an updated context."},{"ref":"StateMachine.Event.html#t:t/1","title":"StateMachine.Event.t/1","type":"type","doc":""},{"ref":"StateMachine.Guard.html","title":"StateMachine.Guard","type":"module","doc":"Guards are functions that help to decide whether it&#39;s allowed to proceed with Event or Transition. They might serve different purposes: Preventing a state machine from getting into a state unless some criteria met Creating event with multiple target states with a single source state Guards should not have any side-effects, cause they are getting run no matter if transition is successful or not, and also to determine the list of possible events for a certain state."},{"ref":"StateMachine.Guard.html#check/2","title":"StateMachine.Guard.check/2","type":"function","doc":"Check runs guards associated with given Event or Transition and returns true if all passed. First argument of the guard is a model, second argument is the context."},{"ref":"StateMachine.Guard.html#prepare/1","title":"StateMachine.Guard.prepare/1","type":"function","doc":"Unifies if and unless guards into a single stream of guards."},{"ref":"StateMachine.Guard.html#t:t/1","title":"StateMachine.Guard.t/1","type":"type","doc":""},{"ref":"StateMachine.Introspection.html","title":"StateMachine.Introspection","type":"module","doc":"Introspection functions allow you to collect and analyze meta information about state machine in runtime. For example, you might want to send the full list of states or events to the front-end, so that appropriate control or scale could be rendered. It is also possible to generate the list of currently allowed events, but be careful, this operation can be expensive if you use guards with side-effects."},{"ref":"StateMachine.Introspection.html#all_events/1","title":"StateMachine.Introspection.all_events/1","type":"function","doc":"Returns a list of all events of the supplied state machine definition."},{"ref":"StateMachine.Introspection.html#all_states/1","title":"StateMachine.Introspection.all_states/1","type":"function","doc":"Returns a list of all states of the supplied state machine definition."},{"ref":"StateMachine.Introspection.html#allowed_events/1","title":"StateMachine.Introspection.allowed_events/1","type":"function","doc":"Returns a list of available events. This function takes a context with populated model, this is necessary to get a current state."},{"ref":"StateMachine.State.html","title":"StateMachine.State","type":"behaviour","doc":"State module provides a structure describing a state in the state machine definition. It stores state name along with various callbacks to run before/after leaving/entering a state. Another purpose of the module is to define a behaviour for state get/setters along with the default implementation. The state get/setters for basic structures and Ecto records are provided out of the box."},{"ref":"StateMachine.State.html#after_enter/1","title":"StateMachine.State.after_enter/1","type":"function","doc":"Private function for running after_enter(state) callbacks."},{"ref":"StateMachine.State.html#after_leave/1","title":"StateMachine.State.after_leave/1","type":"function","doc":"Private function for running after_leave(state) callbacks."},{"ref":"StateMachine.State.html#before_enter/1","title":"StateMachine.State.before_enter/1","type":"function","doc":"Private function for running before_enter(state) callbacks."},{"ref":"StateMachine.State.html#before_leave/1","title":"StateMachine.State.before_leave/1","type":"function","doc":"Private function for running before_leave(state) callbacks."},{"ref":"StateMachine.State.html#get/1","title":"StateMachine.State.get/1","type":"function","doc":"Default implementation of a state getter."},{"ref":"StateMachine.State.html#c:get/1","title":"StateMachine.State.get/1","type":"callback","doc":""},{"ref":"StateMachine.State.html#set/2","title":"StateMachine.State.set/2","type":"function","doc":"Default implementation of a state setter."},{"ref":"StateMachine.State.html#c:set/2","title":"StateMachine.State.set/2","type":"callback","doc":""},{"ref":"StateMachine.State.html#t:t/1","title":"StateMachine.State.t/1","type":"type","doc":""},{"ref":"StateMachine.Transition.html","title":"StateMachine.Transition","type":"module","doc":"Transition module gathers together all of the actions that happen around transition from the old state to the new state in response to an event."},{"ref":"StateMachine.Transition.html#after_/2","title":"StateMachine.Transition.after_/2","type":"function","doc":"Private function for running after_transition callbacks."},{"ref":"StateMachine.Transition.html#before/2","title":"StateMachine.Transition.before/2","type":"function","doc":"Private function for running before_transition callbacks."},{"ref":"StateMachine.Transition.html#finalize/1","title":"StateMachine.Transition.finalize/1","type":"function","doc":"Private function sets status to :done, unless it has failed before."},{"ref":"StateMachine.Transition.html#is_allowed?/2","title":"StateMachine.Transition.is_allowed?/2","type":"function","doc":"Checks if the transition is allowed in the current context. Returns boolean."},{"ref":"StateMachine.Transition.html#run/2","title":"StateMachine.Transition.run/2","type":"function","doc":"Given populated context and Transition structure, sequentially runs all callbacks along with actual state update: before(event) before(transition) before_leave(state) before_enter(state) (state update) after_leave(state) after_enter(state) after(transition) after(event) If any of the callbacks fails, all sequential ops are cancelled."},{"ref":"StateMachine.Transition.html#update_state/1","title":"StateMachine.Transition.update_state/1","type":"function","doc":"Private function for updating state."},{"ref":"StateMachine.Transition.html#t:t/1","title":"StateMachine.Transition.t/1","type":"type","doc":""},{"ref":"StateMachine.Utils.html","title":"StateMachine.Utils","type":"module","doc":""},{"ref":"StateMachine.Utils.html#keyword_splat/3","title":"StateMachine.Utils.keyword_splat/3","type":"function","doc":""},{"ref":"StateMachine.Utils.html#normalize_funciton/2","title":"StateMachine.Utils.normalize_funciton/2","type":"function","doc":""},{"ref":"StateMachine.Utils.html#normalize_function/2","title":"StateMachine.Utils.normalize_function/2","type":"function","doc":""},{"ref":"StateMachine.Validation.html","title":"StateMachine.Validation","type":"module","doc":"StateMachine supports automatic validation on compile time. It makes its best effort to ensure determinism (to some degree). It checks that all used states were previously defined. TODO: Check unreachable states?"},{"ref":"StateMachine.Validation.html#validate_all/1","title":"StateMachine.Validation.validate_all/1","type":"function","doc":""},{"ref":"StateMachine.Validation.html#validate_states_in_transitions/1","title":"StateMachine.Validation.validate_states_in_transitions/1","type":"function","doc":"Validates presense of states used in transitions."},{"ref":"StateMachine.Validation.html#validate_transitions_determinism/1","title":"StateMachine.Validation.validate_transitions_determinism/1","type":"function","doc":"Validates that no more than one unguarded transition is originated in every state."}]